@@ -14,18 +14,26 @@ class systems_analysis:
     '''A big wrapper for conveniently storing a lot of our analysis methods
     '''
 
-    def __init__(self,systems_representations):
+    def __init__(self,systems_representations=None,replicate_distribution=None):
         '''
         Parameters
         ----------
         systems_representations:list, expected=list[array_1,...,array_n-1] where each array is an np.ndarray with shape=(n_frames,n_residues,n_residues)
             Each array should have the shape as described above, where each frame has an adjacency matrix of pairwise
             comparisons between all residues. The only axis that can differ between arrays is the number of frames (n_frames).
+        
+        frame_list: listlike,shape=(data,)
+            A list of integers representing the number of frames present in each replicate. This should be in the order
+            of which the various versions of the system, and replicates where concatenated.
+        
 
         Returns
         -------
         None its an init
 
+        
+
+
         Notes
         -----
         -automatically will convert list of arrays into a feature matrix so its easy to build it in
@@ -41,19 +49,19 @@ class systems_analysis:
         
         self.num_systems=len(systems_representations) #this is useful later on for when we are doing system_specific operations
         self.systems_representations=systems_representations
-        self.indexes = systems_representations[0][0,0,1:]
-        self.feature_matrix=self.replicates_to_featurematrix(systems_representations)
-
-        #this will be updated later and are defined within the functions themselves
-        self.optimal_k_silhouette_labels=None
-        self.optimal_k_elbow_labels=None
-        self.pca_weights=None
-
-
+        self.indexes = systems_representations[0][0, 0, 1:] #bc list then 3d array
+        
+        if replicate_distribution is not None:
+            self.replicate_distribution=replicate_distribution
+            
+        if replicate_distribution is None:
+            self.replicate_distribution=np.arange(0,systems_representations[0].shape[0])
+        
         return
 
+
     #pre-processing
-    def replicates_to_featurematrix(self,arrays)->np.ndarray:
+    def replicates_to_featurematrix(self,arrays=None)->np.ndarray:
         """returns an array formatted for kmeans clustering with scipy
 
         Parameters
@@ -88,6 +96,7 @@ class systems_analysis:
         This results in the expected formatting for scipy's kmeans clustering implementation where each .
         
         """
+        arrays = arrays if arrays is not None else self.systems_representations
         
         #Concatenate arrays and define list to hold reformatted arrays
         try:
@@ -111,15 +120,67 @@ class systems_analysis:
             final_frames.append(flattened)
 
         final_frames = np.vstack(final_frames).astype(np.float32)
+        self.feature_matrix=final_frames#set global var as well
 
         return final_frames
 
+    def extract_hbond_values(self,systems_array,residues,mode="sum"):
+        ''' returns a 1dimensional array of "average" values per entry in the array
+        
+        Parameters
+        ----------
+        systems_array:np.ndarray,shape(n_frames,n_residues,n_residues)
+            An averaged array (or individual frame of a trajectory) that is to be visualized.
+            Since a typical adjacency matrix is NxN observations we will not focus on the multiple
+            frames of the trajectory and will assume an averaged matrix has been provided. Alternatively
+            this function can take a single frame from anywhere in the trajectory if you wish to analyze it
+
+        residues:list,shape=(res_indexes,)
+            A list denoting all of the residue indexes that you would like to analyze pairwise interactions for.
+            This can be two residues meaning you just want to see the pairwise interactions between them or
+            more residues and then you would get *all possible pairwise combinations of theese residues interactions*
+        
+        mode:string,default="sum",
+            A string argument that decides the aggregation metric by which you would like to aggregate every frame. Mean of
+            the residues of the CAR interaction surface for example, would give you the average hydrogen bonding found between all
+            the residues of the car interaction surface and the +1 codon, sum would give you the total net hydrogen bond counts.
+            
+        Returns
+        ----------
+        avg_ta_labels:array,shape=(n_frames,):
+            An array of the same size as the frames of interest except it just contains an average
+            of all the possible pairwise hydrogen bonding interactions of the residues of interest for
+            each frame
+        
+        Notes
+        ----------
+        This is actually a more powerful function than it may appear at first because if you only,
+        say your pca found a few pairwise comparisons be incredibly important, well now we can isolate
+        for just thoose frames and see which is best.
+
+        '''
+
+        # we always assume it comes with indixes but filter res accounts for this
+        # this note in relation to the lines after filter res in the clauses
+
+        if mode == "average":
+            systems_array,residues
+            filtered_t_a_array=filtered_t_a_array[:,1:,1:]        
+            ta_labels = np.mean(np.triu(filtered_t_a_array, k=1), axis=(1, 2)) #accounting for symmetry
+        
+        if mode == "sum":      
+            systems_array,residues
+            filtered_t_a_array=filtered_t_a_array[:,1:,1:]
+            ta_labels = np.sum(np.triu(filtered_t_a_array, k=1), axis=(1, 2)) #accounting for symmetry
+        
+        return ta_labels
+
     #Analyses
-    def cluster_system_level(self,outfile_path, max_clusters=None,data=None):
+    def cluster_system_level(self,outfile_path=None, max_clusters=None,data=None,k=None):
         '''
         Parameters
         ----------
-        data:np.ndarray,shape=(n_sample,n_features),
+        data:np.ndarray,shape=(n_sample,featuresures),
             A feature matrix of any kind, hopefully one provided from the rest of the pipeline but in theory, this is 
             just a scikit learn wrapper so you can plug anything you want really
 
@@ -129,11 +190,15 @@ class systems_analysis:
         outfile_path:str,default=os.getcwd()
             The path to where we would like to save the outputted labels (frame assignments of K-means)
         
-        data:arraylike,shape=(n_samples,n_features)
+        data:arraylike,shape=(n_samples,featuresures)
             Ideally this is the feature matrix provided as input at the top of the workflow but, its provided as a parameter incase
             you'd like to use theese in your own way.
         
-
+        k:int,default=None
+            If you have a happen to have a specific number of k you would like to keep its data for you can do this
+            as otherwise you end up defaulting to either sillohuette score or elbow (inertia) evlauated optimal K. It does
+            save every array inbetween for thoroughness.
+        
         Returns
         ----------
         (optimal_k_silhouette_labels,optimal_k_elbow_labels,centers_sillohuette,centers_elbow):tuple,shape=(4)
@@ -144,23 +209,37 @@ class systems_analysis:
             
         Notes
         -------
-
-
+        We do assume you have atleast 10 frames worth of data... clustering any less than that
+        is a little out of the scope of this simple euclidean distance K-means clustering implementation.
         
         Examples
         ---------
 
         
         '''
+
         max_clusters=max_clusters if max_clusters is not None else 10
         data = data if data is not None else self.feature_matrix
         outfile_path=outfile_path if outfile_path is not None else os.getcwd()
-        optimal_k_silhouette_labels,optimal_k_elbow_labels,centers_sillohuette,centers_elbow=self.preform_clust_opt(outfile_path=outfile_path,data=data,max_clusters=max_clusters)
+        k=k if k is not None else None
 
+        if k is None:
+            optimal_k_silhouette_labels,optimal_k_elbow_labels,centers_sillohuette,centers_elbow=self.preform_clust_opt(outfile_path=outfile_path,data=data,max_clusters=max_clusters)
+        
+        if k is not None:
+            kmeans = KMeans(n_clusters=k, init='random', n_init=k, random_state=0)
+            kmeans.fit(data)
+            cluster_centers, inertia, cluster_labels = kmeans.cluster_centers_,kmeans.inertia_,kmeans.labels_
+            return cluster_labels,cluster_centers
+            
 
         return optimal_k_silhouette_labels,optimal_k_elbow_labels,centers_sillohuette,centers_elbow
     
-    def reduce_systems_representations(self,feature_matrix=None,method=None,n_components=None,min_dist=None,n_neighbors=None):
+    def reduce_systems_representations(self,feature_matrix=None,
+                                        method=None,
+                                        n_components=None,
+                                        min_dist=None,
+                                        n_neighbors=None):
         '''
         Parameters
         ----------
@@ -184,6 +263,7 @@ class systems_analysis:
         n_neighbors:int,default=900
             Another UMAP-specific parameter. It determines the number of neighboring points used in 
             local approximations of the manifold. Larger values result in more global structure being preserved.
+ 
 
 
         
@@ -224,8 +304,15 @@ class systems_analysis:
 
         
         '''
+        
+        if feature_matrix is not None:
+            feature_matrix = feature_matrix 
+        if self.feature_matrix is not None:
+            feature_matrix=self.feature_matrix
+        if self.feature_matrix is None:
+            self.replicates_to_featurematrix()
+            feature_matrix=self.feature_matrix
 
-        feature_matrix = feature_matrix if feature_matrix is not None else self.feature_matrix
         n_components=n_components if n_components is not None else 2
         method = method if method is not None else 'PCA'
         min_dist = min_dist if min_dist is not None else .5
@@ -248,13 +335,7 @@ class systems_analysis:
         elif method != 'PCA' and method != 'UMAP':
             print('No valid method supplied for dimensional reduction ')
         
-    def cluster_embeddingspace(self,outfile_path=None,feature_matrix=None,n=None,max_clusters=None,elbow_or_sillohuette=None):
-        '''
-        This has been depreciated plz ignore lol
-        
-        '''
-        
-        
+    def cluster_embeddingspace(self, reduced_coordinates=None,outfile_path=None,num_systems=None,val_metric=None):
         '''A function for looking at conformational states in embedding space
 
         Parameters
@@ -262,7 +343,7 @@ class systems_analysis:
         outfilepath:str
             path to save
 
-        featurematrix:np.Ndarray,default=self.featurematrix
+        reduced_coordinates:np.Ndarray,default=self.reduced_coordinates,shape=
             A feature matrix to be used for analysis
         
         n:int,default=2
@@ -271,83 +352,67 @@ class systems_analysis:
         max_clusters:int,default=10
             The defualt number
 
+        val_metric:str,default='sillohuete'
+            The validation metric you would like to use for picking an "ideal number of clusters'
+            
 
 
         Returns
         -------
+        candidate_states_per_system:list,shape=(n_systems,)
+            A list where each entry corresponds to a system. Each entry is a tuple of
+            (labels, centers) representing the cluster assignments for that system and
+            the cluster centers determined by the chosen validation metric.
 
 
 
         Notes
         -----
+        This function runs clustering for each system independently, based on the 
+        dimensionality-reduced feature space. Both elbow and sillohuete methods are 
+        computed internally, but only the method specified in val_metric is returned.
+        Returned states are meant for downstream analysis such as transition probability 
+        calculations, replicate maps, or visualization.
 
 
 
         Examples
         --------
+        >>> systems_clusters=self.cluster_embeddingspace(reduced_coordinates=X_pca,
+        ...                                               outfile_path='/results/',
+        ...                                               num_systems=2,
+        ...                                               val_metric='sillohuete')
+        >>> print(len(systems_clusters))
+        2
 
         '''
 
         outfile_path = outfile_path if outfile_path is not None else os.getcwd()
-        feature_matrix = feature_matrix if feature_matrix is not None else self.feature_matrix
-        n=n if n is not None else 2
-        max_clusters=max_clusters if max_clusters is not None else 10
-        elbow_or_sillohuette=elbow_or_sillohuette if elbow_or_sillohuette is not None else 'sillohuette'
+        num_systems = num_systems if num_systems is not None else self.num_systems
+        val_metric=val_metric if val_metric is not None else 'sillohuette'
         
-        X_pca,weights,explained_variance_ratio_=self.run_PCA(feature_matrix,n)
+        if reduced_coordinates is None:
+            reduced_coordinates,_,_=self.reduce_systems_representations()
+
         
         #grab the number of rows we need and then iterate through X_pca run kmeans and visualize using our initial values
-        rows_per_system=X_pca.shape[0]/self.num_systems
-
-        index=0
+        individual_systems=np.array_split(reduced_coordinates, num_systems,axis=0)
+        print(individual_systems[0].shape)
+        candidate_states_per_system=[]
+        for i in individual_systems:
+            optimal_k_silhouette_labels,optimal_k_elbow_labels,centers_sillohuette,centers_elbow=self.preform_clust_opt(outfile_path,data=i)
+            if val_metric=='sillohuette':
+                candidate_states_per_system.append((optimal_k_silhouette_labels,centers_sillohuette))
+            if val_metric=='elbow':
+                candidate_states_per_system.append((optimal_k_elbow_labels,centers_elbow))
 
 
-        for i in range(self.num_systems):
-            # padding before and after so we can still visualize them all on the same scale
-            before = index
-            full_path=outfile_path+f"_lowdimclust_sys_{i+1}"
-            current_system_frames = X_pca[index:index + int(rows_per_system), :]
-           
-            optimal_k_silhouette_labels,optimal_k_elbow_labels,centers_sillohuette,centers_elbow=self.preform_clust_opt(outfile_path=full_path,data=current_system_frames,max_clusters=max_clusters)
-       
-            if elbow_or_sillohuette == 'sillohuette':
-                corelation_dataframe = self.create_pearsontest_for_kmeans_distributions(optimal_k_silhouette_labels,current_system_frames,centers_sillohuette)
-                print(f"corelation_dataframe for system {i}\n{corelation_dataframe}")
-                after = X_pca.shape[0] - (index + len(optimal_k_silhouette_labels))
+        return candidate_states_per_system
 
-                #this is where we use the previous spot for markings
-                labels_filled = np.concatenate([
-                np.full(before,(np.max(optimal_k_silhouette_labels)+1)),                      # padding before
-                optimal_k_silhouette_labels,              # actual cluster labels
-                np.full(after,(np.max(optimal_k_silhouette_labels)+1))                        # padding after
-                ])
-
-            elif elbow_or_sillohuette == 'elbow':
-                corelation_dataframe = self.create_pearsontest_for_kmeans_distributions(optimal_k_elbow_labels,current_system_frames,centers_elbow)
-                print(f"corelation_dataframe for system {i}\n{corelation_dataframe}")
-                after = X_pca.shape[0] - (index + len(optimal_k_elbow_labels))
-
-                #this is where we use the previous spot for markings
-                labels_filled = np.concatenate([
-                np.full(before, (np.max(optimal_k_elbow_labels)+1)),                      # padding before
-                optimal_k_elbow_labels,              # actual cluster labels
-                np.full(after, (np.max(optimal_k_elbow_labels)+1))                        # padding after
-                ])
-
-            
-
-            visualize_reduction(X_pca,
-                            color_mappings=labels_filled,
-                            savepath=full_path,
-                            title=f'optimal sillohuette clustering in embedding space for system {i}',
-                            custom=False)
-            index+=int(rows_per_system)
-        
-        self.pca_weights=weights
-
-        return 
-    
     def create_pearsontest_for_kmeans_distributions(self,labels,coordinates,cluster_centers):
+        '''
+        Currently depreciated
+        '''
         '''A function that is meant for the 
 
         Parameters
@@ -355,7 +420,7 @@ class systems_analysis:
         labels:listlike
             A list or array of labels that tell us which cluster each sample belongs to
 
-        coordinates:array,shape=(n_samples,n_features)
+        coordinates:array,shape=(n_samples,featuresures)
             An array which tells us the coordinates of each sample so we can form distributions from them and run statistical tests
             (pearson corellation coefficient)
         
@@ -427,7 +492,7 @@ class systems_analysis:
         Parameters
         ----------
 
-        weights : np.ndarray, shape = (n_components, n_features)
+        weights : np.ndarray, shape = (n_components, featuresures)
             PCA component loadings (rows = components, columns = features). If None, this function
             calls `reduce_systems_representations()` to compute PCA (default n=2) and uses the
             returned `weights`.
@@ -494,195 +559,124 @@ class systems_analysis:
         
         return dataframe
 
-    def create_contour_plot(self,outfile_path=None,feature_matrix=None):
-        '''
-        Parameters
-        ----------
-
-        Notes
-        -----
-
-        Examples
-        --------
 
 
-        Returns
-        -------
-        
-        '''
-        feature_matrix=feature_matrix if feature_matrix is not None else self.feature_matrix
-        outfile_path = outfile_path if outfile_path is not None else os.getcwd()
-        
-        X_pca,weights,explained_variance_ratio_=self.run_PCA(feature_matrix,2)
-        
-        import seaborn as sns
-        import matplotlib.pyplot as plt
-
-
-
-        '''
-        Since we are not explicitly setting an h seaborn automatically picks one using scotts rule or silvermans rule
-        the param would be bandwidth and can be adjusted using bw_adjust=.05 for instance creating narrower kernels
-
-        bandwidth-> spread of the KDE (shape of the landscape)
-
-        levels-> controls where to draw the contour lines (which KDE values)
-            We arent setting levels so it can really be whatever at that point
-            
-        thresh-> hides regiosn below this KDE value (cut-off for noise)
-
-        fill=true fills betweent he contours
-
-        cbar = true
-
-
-        '''
-        sns.kdeplot(
-            x=X_pca[:, 0],  # PC1
-            y=X_pca[:, 1],  # PC2
-            fill=True,      # shaded contours
-            cmap="cviridis",
-            thresh=0,#only plots regions where values are greater than some threshold 
-            cbar=True
-        )
-
-
-
-
-
-        
-
-        return
-
-    def cluster_individual_systems_in_embeddingspace(self, reduced_data=None, frames_per_sys=None, num_systems=None):
-        '''cluster individual systems in embedding space in order to identify potential conformations
-
-        Parameters
-        ----------
-        reduced_data:arraylike,shape=(n_samples,2)
-            Data that has been reduced using a dimensional reduction method such as PCA, UMAP, TSNE, etc.
-            If None, the function will use the UMAP reduction from self.reduce_systems_representations().
-
-        frames_per_sys:int
-            The number of frames contained per one version of the system you are exploring.
-            If None, the function will use the number of frames from the first system in self.systems_representations.
-        
-        num_systems:int
-            The number of different systems (e.g., simulations, replicates) being analyzed.
-            If None, the function will use the value stored in self.num_systems.
-
-        Returns
-        -------
-        results:list,shape=(n_systems,)
-            Returns the results as a list of arrays that you can iterate through, where each array
-            contains the cluster labels for a single system, padded with a high value.
-
-        Notes
-        -----
-        This function assumes you have the same number of frames per system. It first performs 
-        clustering on each system individually and then pads the resulting labels to the original
-        total size, making them easy to visualize against the full embedding space.
-
-        Examples
-        --------
-     
-        '''
-
-        
-        if reduced_data is not None :
-            reduced_data = reduced_data 
-
-        if reduced_data is None:
-            x_pca,_,_ = self.reduce_systems_representations(method='PCA')
-            reduced_data=x_pca
-            
-        num_systems = num_systems if num_systems is not None else self.num_systems
-        
-        if frames_per_sys is None:
-            frames_per_sys = self.systems_representations[0].shape[0]
-        
-        iterator = 0
-        results = []
-         
-        
-        for i in range(num_systems):
-
-            current_sys_data = reduced_data[iterator : iterator + frames_per_sys, :]
-            optimal_k_silhouette_labels, _, _, _ = self.preform_clust_opt(outfile_path=f"embeddingspace_system_{i}_clust", data=current_sys_data)
-            empty_sys = np.full(reduced_data.shape[0], 10)
-            empty_sys[iterator : iterator + frames_per_sys] = optimal_k_silhouette_labels
-            
-            results.append(empty_sys)
-            
-            iterator += frames_per_sys
-        
-        return results
-    
     #Algorithm wrappers 
-    def preform_clust_opt(self,outfile_path, max_clusters=None, data=None):
+    def preform_clust_opt(self,outfile_path, max_clusters=None, data=None,k=None):
         '''
         Parameters
         ----------
-        data:np.ndarray,shape=(n_sample,n_features),
-            A feature matrix of any kind, hopefully one provided from the rest of the pipeline but in theory, this is 
-            just a scikit learn wrapper so you can plug anything you want really
-        
-        outfile_path:str,default=os.getcwd()
+        data : np.ndarray, shape = (n_samples, featuresures)
+            A feature matrix of any kind (ideally produced earlier in this pipeline).
+            This is just a scikit-learn wrapper, so you can plug anything you want really.
+            If None, defaults to `self.feature_matrix`.
         
+        outfile_path : str, default = os.getcwd()
+            Path prefix where per-K label arrays will be saved via `np.save`.
+            A file is written for each tested K as
+            "<outfile_path>kluster_labels_{K}clust.npy".
+            (If you don’t pass a trailing separator, this will be concatenated verbatim.)
+
+        max_clusters : int, default = 10
+            When `k is None`, we grid-search KMeans for K in [2, ..., max_clusters]
+            to compute inertia (elbow) and sillohuette scores.
+
+        k : int or None, default = None
+            If provided, we skip optimization and run a single KMeans with exactly `k`
+            clusters (useful when you already know the K you want to keep).
 
         Returns
         ----------
-        
+        If `k is not None`:
+            (cluster_labels, cluster_centers) : tuple
+                cluster_labels : np.ndarray, shape = (n_samples,)
+                    The assignment of each sample to its cluster.
+                cluster_centers : np.ndarray, shape = (k, featuresures)
+                    The KMeans centroids for the requested K.
+
+        If `k is None` (optimization path):
+            (optimal_k_silhouette_labels, optimal_k_elbow_labels, centers_sillohuette, centers_elbow) : tuple
+                optimal_k_silhouette_labels : np.ndarray, shape = (n_samples,)
+                    Labels for the K that maximizes sillohuette score over the tested range.
+                optimal_k_elbow_labels : np.ndarray, shape = (n_samples,)
+                    Labels for the K chosen by the elbow (inertia) heuristic.
+                centers_sillohuette : np.ndarray, shape = (K_sillohuette, featuresures)
+                    Centroids corresponding to the sillohuette-optimal labels.
+                centers_elbow : np.ndarray, shape = (K_elbow, featuresures)
+                    Centroids corresponding to the elbow-optimal labels.
+
         Notes
         ----------
-        
+        - KMeans config here is `init='random'`, `n_init=k` (or `n_init=K` inside the loop),
+        and `random_state=0` for reproducibility—kept consistent with the rest of this code.
+        - Validation metrics:
+            * **Sillohuette** (via `sklearn.metrics.silhouette_score`) favors compact, well-separated clusters.
+            * **Elbow** uses raw inertia (within-cluster SSE) to pick a knee in the curve.
+        The helper functions `plot_sillohette_scores` and `plot_elbow_scores` (from `mdsa_tools.Viz`)
+        are called to optionally save plots and to return the selected K values.
+        - Side effects: for each tested K in `[2, max_clusters]`, this writes labels to disk using
+        `np.save(f"{outfile_path}kluster_labels_{K}clust", labels)`. Numpy will append the `.npy`
+        extension if not present.
+
         Examples
         ----------
-        
+        >>> # 1) Let the function pick K by sillohuette and elbow
+        >>> opt_sil_labels, opt_elb_labels, sil_centers, elb_centers = \
+        ...     sa.preform_clust_opt(outfile_path='/results/', max_clusters=12, data=sa.feature_matrix)
+        >>>
+        >>> # 2) Run a fixed-K clustering (no optimization)
+        >>> labels_k5, centers_k5 = sa.preform_clust_opt(outfile_path='/results/', data=sa.feature_matrix, k=5)
         '''
+        
         data = data if data is not None else self.feature_matrix
         outfile_path = outfile_path if outfile_path is not None else os.getcwd()
         max_clusters = max_clusters if max_clusters is not None else 10
+        k=k if k is not None else None
         
-        
-        #keeping track of our scores 
-        inertia_scores,silhouette_scores,all_labels,centers = [],[],[],[]
-        cluster_range = range(2, max_clusters+1)
-
-
-        for k in cluster_range:
-            kmeans = KMeans(n_clusters=k, init='random', n_init=k, random_state=0) #we set
-            kmeans.fit(data) #fit data and now we have everything transformed
+        if k is not None:
+            kmeans = KMeans(n_clusters=k, init='random', n_init=k, random_state=0)
+            kmeans.fit(data)
             cluster_centers, inertia, cluster_labels = kmeans.cluster_centers_,kmeans.inertia_,kmeans.labels_
-            sil_score = silhouette_score(data, cluster_labels)
+            return cluster_labels,cluster_centers
+        
+        if k is None:
+            #keeping track of our scores 
+            inertia_scores,silhouette_scores,all_labels,centers = [],[],[],[]
+            cluster_range = range(2, max_clusters+1)
+
+            for k in cluster_range:
+                kmeans = KMeans(n_clusters=k, init='random', n_init=k, random_state=0) #we set
+                kmeans.fit(data) #fit data and now we have everything transformed
+                cluster_centers, inertia, cluster_labels = kmeans.cluster_centers_,kmeans.inertia_,kmeans.labels_
+                sil_score = silhouette_score(data, cluster_labels)
+                
+                centers.append(cluster_centers)
+                inertia_scores.append(inertia)
+                silhouette_scores.append(sil_score)
+                all_labels.append(cluster_labels)
+
+
+                np.save(f"{outfile_path}kluster_labels_{k}clust",cluster_labels)
+
             
-            centers.append(cluster_centers)
-            inertia_scores.append(inertia)
-            silhouette_scores.append(sil_score)
-            all_labels.append(cluster_labels)
+            #so we save unless your calling this specific optimization
+            from mdsa_tools.Viz import plot_elbow_scores,plot_sillohette_scores
 
+            
+            optimal_sillohuette=plot_sillohette_scores(cluster_range,silhouette_scores,outfile_path)
+            optimal_elbow=plot_elbow_scores(cluster_range,inertia_scores,outfile_path)
 
-            np.save(f"{outfile_path}kluster_labels_{k}clust",cluster_labels)
+            #print(f"\nsize of labels:{len(all_labels)} ,optimal_elbow: {optimal_elbow}:optimal_sillohuette {optimal_sillohuette}")
 
-        
-        #so we save unless your calling this specific optimization
-        from mdsa_tools.Viz import plot_elbow_scores,plot_sillohette_scores
+            # Now you can return optimal k values
+            
+            optimal_k_silhouette_labels = all_labels[optimal_sillohuette-2] 
+            optimal_k_elbow_labels = all_labels[optimal_elbow-2]
+            centers_sillohuette = centers[optimal_sillohuette-2] 
+            centers_elbow = centers[optimal_elbow-2] 
 
-        
-        optimal_sillohuette=plot_sillohette_scores(cluster_range,silhouette_scores,outfile_path)
-        optimal_elbow=plot_elbow_scores(cluster_range,inertia_scores,outfile_path)
-
-        #print(f"\nsize of labels:{len(all_labels)} ,optimal_elbow: {optimal_elbow}:optimal_sillohuette {optimal_sillohuette}")
-
-        # Now you can return optimal k values
-        
-        optimal_k_silhouette_labels = all_labels[optimal_sillohuette-2] 
-        optimal_k_elbow_labels = all_labels[optimal_elbow-2]
-        centers_sillohuette = centers[optimal_sillohuette-2] 
-        centers_elbow = centers[optimal_elbow-2] 
-
-        
-        return optimal_k_silhouette_labels,optimal_k_elbow_labels,centers_sillohuette,centers_elbow
+            
+            return optimal_k_silhouette_labels,optimal_k_elbow_labels,centers_sillohuette,centers_elbow
     
     def run_PCA(self,feature_matrix,n):
         '''small function for running principal components analysis
@@ -718,146 +712,17 @@ class systems_analysis:
 
         pca=PCA(n_components=n)
         pca.fit(feature_matrix)
-        new_values=pca.components_ 
         X_pca = pca.transform(feature_matrix)
         weights = pca.components_
-        explained_variance_ratio_ = pca.explained_variance_ratio_
+        explained_variances = pca.explained_variance_ratio_
 
         print("X_pca shape (new data):",X_pca.shape)
-        print(f"the total explained variance ratio is {np.sum(explained_variance_ratio_)}")
-        print(f"the total explained variance of PC's is {explained_variance_ratio_}")
+        print(f"the total explained variance{np.sum(explained_variances)}")
+        print(f"the total explained variance of PC's is {explained_variances}")
         print("weights shape:", weights.shape) 
         
-        return X_pca,weights,explained_variance_ratio_
+        return X_pca,weights,explained_variances
 
-class MSM_Modeller():
-
-    def __init__(self,labels,frame_list):
-        self.labels=labels if labels is not None else None
-        self.frame_list=frame_list if frame_list is not None else frame_list
-
-        self.transition_probability_matrix=None
-        self.lag=None
-
-    def create_transition_probability_matrix(self,labels=None,frame_list=None,lag=None):
-        '''Create probability matrix from input data (returns, and updates class attribute)
-
-        Parameters
-        ----------
-        labels:arraylike,shape=(n_labels,)
-            A list of labels pertaining to frames of molecular dynamics trajectories assigned particular substates
-
-        frame_list: listlike,shape=(data,)
-            A list of integers representing the number of frames present in each replicate. This should be in the order
-            of which the various versions of the system, and replicates where concatenated. 
-
-        
-        Returns
-        -------
-        transition_probability_matrix:arraylike;shape=(n_states+1,n_states+1)
-            A transition probability matrix created from the list of labels. Diagonals indicate
-            if it is likely to stay in the same state and off diagonals mark probabilities of transitions
-
-
-
-        
-        Notes
-        -----
-        Much in the spirit of our original matrices the first row and column of theese matrices contain
-        indexes mainly for ease of use and manipulation. Yes, in theory pandas dataframes could streamline this process
-        but, numpy arrays are just that much more efficient in most use cases,
-
-
-
-        Examples
-        --------
-
-        
-
-        '''
-
-
-        labels=labels if labels is not None else self.labels
-        frame_list=frame_list if frame_list is not None else self.frame_list
-        lag=lag if lag is not None else 1
-
-        #extract unique states and initiate transiiton probability matrix
-        unique_states=np.unique(labels)
-        number_of_states=len(unique_states)
-        transtion_prob_matrix=np.zeros(shape=(number_of_states,number_of_states))
-        
-        iterator=0
-        for trajectory_length in frame_list: # iterate through 
-            current_trajectory=labels[iterator:iterator+trajectory_length]
-            iterator=iterator+trajectory_length #update this 
-
-            for i in range(current_trajectory.shape[0]-lag):
-                current_state=current_trajectory[i]
-                next_state = current_trajectory[i+lag]
-                transtion_prob_matrix[current_state, next_state] += 1
-
-        row_sums = transtion_prob_matrix.sum(axis=1, keepdims=True)
-        transition_probs = transtion_prob_matrix / row_sums
-
-        final_transition_prob_matrix=np.zeros(shape=(number_of_states+1,number_of_states+1))
-        final_transition_prob_matrix[1:,1:]=transition_probs
-        final_transition_prob_matrix[0,1:],final_transition_prob_matrix[1:,0]=unique_states,unique_states
-
-        self.transition_probability_matrix=final_transition_prob_matrix
-        print(final_transition_prob_matrix)
-
-
-        return final_transition_prob_matrix
-    
-
-    def evaluate_Chapman_Kolmogorov(self,transition_probability_matrix=None,n=None,labels=None,original_lag=None):
-        '''evaluate if the chapman kolmogorov test evaluates to true
-
-        Parameters
-        ----------
-        n:int,default=4
-            The original number of lags we used to compute the transition probability matrix
-        
-        transition_proability_matrix:arraylike,shape=(n_states+1,n_states+1),
-
-        n:int,default=4
-            The time lag we are using to compute our labels
-
-        labels:arraylike,default=self.labels
-            The list of labels we are using for the labeling of data from trajectories. 
-        
-        original_lag:int:default=1
-
-
-        Notes
-        -----
-
-        
-        Returns
-        -------
-
-
-
-        Examples
-        --------
-        
-        
-        '''
-
-        transition_probability_matrix=transition_probability_matrix if transition_probability_matrix is not None else self.create_transition_probability_matrix()
-        original_lag=original_lag if original_lag is not None else 1
-        n = n if n is not None else 4
-        labels=labels if labels is not None else self.labels
-
-        transition_prob_data=transition_probability_matrix[1:,1:]
-        post_timestep_data=np.linalg.matrix_power(transition_prob_data,n)
-        transition_probability_matrix[1:,1:]=post_timestep_data
-
-        total_lag=original_lag+n
-        matrix_from_total_lag = self.create_transition_probability_matrix(lag=total_lag)
-        diff=matrix_from_total_lag[1:,1:]-transition_probability_matrix[1:,1:]
-        frob = np.linalg.norm(diff, ord='fro')
-        return frob
 
 
 if __name__ == '__main__':
