@@ -1,16 +1,128 @@
 import numpy as np
 import os
 import matplotlib.pyplot as plt
-from matplotlib.colors import ListedColormap
-from matplotlib.colors import Normalize
 import matplotlib.cm as cm
 import pycircos.pycircos as py 
 import seaborn as sns
+from matplotlib.colors import BoundaryNorm
+import matplotlib.colors as mcolors
+from matplotlib.cm import ScalarMappable
 
+#Miscellaneous tools
+def add_continuous_colorbar(scatter, labels, cbar_label=None, ax=None, cmap=None,
+                            extend="neither", format=None):
+    """
+    Continuous colorbar alternative to `add_custom_colorbar`.
+    Works with numeric and non-numeric labels.
+    """
+    if ax is None:
+        ax = plt.gca()
 
+    cmap_obj = plt.get_cmap(cmap or cm.inferno)
+
+    # Build numeric values
+    if labels is None:
+        n = scatter.get_offsets().shape[0]
+        vals = np.arange(n, dtype=float)
+    else:
+        vals = np.asarray(labels)
+
+    if not np.issubdtype(vals.dtype, np.number):
+        _, inv = np.unique(vals, return_inverse=True)
+        vals = inv.astype(float)
+
+    finite = np.isfinite(vals)
+    if not finite.any():
+        vmin, vmax = 0.0, 1.0
+    else:
+        vmin = float(np.nanmin(vals[finite]))
+        vmax = float(np.nanmax(vals[finite]))
+        if vmin == vmax:
+            pad = 0.5 if vmin == 0 else 0.01 * abs(vmin)
+            vmin, vmax = vmin - pad, vmax + pad
+
+    norm = mcolors.Normalize(vmin=vmin, vmax=vmax)
+
+    # Make sure the scatter is actually using this data/norm/cmap
+    scatter.set_norm(norm)
+    scatter.set_cmap(cmap_obj)
+    scatter.set_array(vals)
+
+    mappable = ScalarMappable(norm=norm, cmap=cmap_obj)
+    mappable.set_array(vals)
+    cbar = plt.colorbar(mappable, ax=ax, extend=extend, format=format)
+    if cbar_label:
+        cbar.set_label(cbar_label, fontsize=10)
+    return cbar
+
+def add_custom_colorbar(scatter, labels, cbar_label=None, ax=None, cmap=None):
+    if ax is None:
+        ax = plt.gca()
+
+    labels = np.asarray(labels)
+    uniques, label_ids = np.unique(labels, return_inverse=True)
+    N = len(uniques)
+
+    cmap = cmap if cmap is not None else cm.inferno
+    bounds = np.arange(-0.5, N + 0.5, 1)  # [-0.5, 0.5], [0.5, 1.5]
+    norm = BoundaryNorm(bounds, cmap.N)
+
+    scatter.set_cmap(cmap)
+    scatter.set_array(label_ids)
+    scatter.set_norm(norm)
+
+    cbar = plt.colorbar(scatter, ax=ax, boundaries=bounds,
+                        ticks=np.arange(N), pad=0.02, shrink=0.8)
+    cbar.set_label(cbar_label or 'Value', fontsize=10)
+
+    if N>100:
+        tick_positions = np.arange(N)[::10]         # every 10th tick
+        tick_labels   = [str(u) for u in uniques][::10]
+        cbar.set_ticks(tick_positions)
+        cbar.set_ticklabels(tick_labels)
+        return cbar
+
+    cbar.set_ticklabels([str(u) for u in uniques])
+
+    return cbar
+
+def set_ticks(ax=None):
+    """
+    Set x and y ticks for an axis. If the axis range is greater than 100,
+    ticks are placed every 10 units. Otherwise, the default tick locator
+    from Matplotlib is preserved.
+
+    Parameters
+    ----------
+    ax : matplotlib.axes.Axes, default=None
+        Axis to apply tick settings. Defaults to current axis.
+
+    Returns
+    -------
+    None
+        Modifies the axis in place.
+    """
+    if ax is None:
+        ax = plt.gca()
+
+    xmin, xmax = ax.get_xlim()
+    if xmax - xmin > 100:
+        ax.set_xticks(np.arange(np.floor(xmin), np.ceil(xmax) + 1, 10))
+
+    ymin, ymax = ax.get_ylim()
+    if ymax - ymin > 100:
+        ax.set_yticks(np.arange(np.floor(ymin), np.ceil(ymax) + 1, 10))
+
+    return
 
 #Replicate maps
-def replicatemap_from_labels(labels,frame_list,savepath=None,title=None, xlabel=None, ylabel=None) -> np.ndarray:
+def replicatemap_from_labels(labels,frame_list,
+                            savepath=None,
+                            title=None,
+                            xlabel=None, ylabel=None,
+                            cbar_label=None,
+                            cmap=None,
+                             ) -> np.ndarray:
     '''returns an array consisting of a re-formatted list of labels through which to view a set.
 
     Parameters
@@ -53,33 +165,49 @@ def replicatemap_from_labels(labels,frame_list,savepath=None,title=None, xlabel=
 
     '''
 
+    cmap=cmap if cmap is not None else cm.magma_r
+
     savepath=savepath if savepath is not None else os.getcwd()
 
 
     iterator=0
     final_coordinates=[]
 
+
+
     for i in range(len(frame_list)):
-        current_replicate_coordinates=np.full(shape=(frame_list[i],),fill_value=i+1) #make list of 11111 then 22222 for each rep
-        frame_positions=np.arange(1,frame_list[i]+1)
-        frame_values=labels[iterator:iterator+frame_list[i]]
+
+        current_frame_length=frame_list[i]
+
+        current_replicate_coordinates=np.full(shape=(current_frame_length,),fill_value=i) #make list of 11111 then 22222 for each rep
+        frame_positions=np.arange(current_frame_length)
+        frame_values=labels[iterator:iterator+current_frame_length]
+        (frame_values.shape)
         replicate_block = np.stack([current_replicate_coordinates, frame_positions, frame_values], axis=1)
         final_coordinates.append(replicate_block)
-        iterator+=frame_list[i]
+        iterator+=current_frame_length
     
     final_coordinates = np.vstack(final_coordinates)
-
-    y_spacing_factor = 10 
-    x_spacing_factor = 10
-
-    plt.scatter(
+    
+    y_spacing_factor = 1
+    x_spacing_factor = 1
+    
+    
+    scatter=plt.scatter(
                 x=final_coordinates[:,1] * x_spacing_factor,
                 y=final_coordinates[:,0] * y_spacing_factor,
                 c=final_coordinates[:,2],
                 s=1,
                 marker='s',
-                cmap=cm.plasma_r)
+                cmap=cmap,
+                alpha=1)
     
+    #cbar and cbar ticks
+    if np.unique(final_coordinates).shape[0]>=1000:
+        add_continuous_colorbar(scatter,final_coordinates[:,2],cbar_label,plt.gca())
+    if np.unique(final_coordinates).shape[0]<1000:
+        add_custom_colorbar(scatter,final_coordinates[:,2],cbar_label,plt.gca())
+
     #personal preferences
     plt.grid(visible=False)
     currentaax=plt.gca()
@@ -91,19 +219,7 @@ def replicatemap_from_labels(labels,frame_list,savepath=None,title=None, xlabel=
     ax.set_title('Clusters per frame', fontsize=20, weight='bold', family='monospace', style='italic')
     n_reps = int(final_coordinates[:, 0].max())
     n_frames = int(final_coordinates[:, 1].max())
-
-    #Setting Ticks
-    x_ticks_labels = np.arange(0, n_frames+10, 10)
-    x_ticks_locations = x_ticks_labels * x_spacing_factor
-    ax.set_xticks(x_ticks_locations)
-    ax.set_xticklabels([str(i) for i in x_ticks_labels],fontsize=8)
-
-
-    y_ticks_labels = np.arange(0, n_reps+10,10)
-    y_ticks_locations = y_ticks_labels * y_spacing_factor
-    ax.set_yticks(y_ticks_locations)
-    ax.set_yticklabels([str(i) for i in y_ticks_labels], fontsize=8)
-
+    
     if title:
         plt.title(title)
     if xlabel:
@@ -120,77 +236,85 @@ def replicatemap_from_labels(labels,frame_list,savepath=None,title=None, xlabel=
     return 
 
 #K-means Cross-validation metrics
-def plot_sillohette_scores(cluster_range, silhouette_scores, outfile_path="sillohette_method.png"):
-    '''quickly plot sillouhette scores afte running kmeans
+def plot_sillohette_scores(cluster_range, silhouette_scores, outfile_path="sillohette_method.png",
+                           title=None, xlabel=None, ylabel=None):
+    """
+    Plot silhouette scores over k and mark the maximum.
+
     Parameters
     ----------
+    cluster_range : array-like
+        k values evaluated.
+
+    silhouette_scores : array-like
+        Silhouette score per k.
+
+    outfile_path : str, default="sillohette_method.png"
+        Path prefix where figure is saved (suffix 'sillohuette_plot' is appended).
+
+    title, xlabel, ylabel : str or None
+        Optional title/axis labels.
 
     Returns
     -------
-
-    Notes
-    -----
-
-
-    Examples
-    --------
-
-    '''
-    # Optimal k is where the silhouette score is highest
-
+    optimal_k_sil : int
+        k with max silhouette score.
+    """
     optimal_k_sil = cluster_range[np.argmax(silhouette_scores)]#return index 
-
     # Plot Silhouette Scores
     plt.figure(figsize=(8, 5))
     plt.plot(cluster_range, silhouette_scores, marker='o', linestyle='-')
     plt.axvline(optimal_k_sil, color='red', linestyle='--', linewidth=2, label=f'Optimal k = {optimal_k_sil}')
-    
-    plt.xlabel('Number of Clusters (k)')
-    plt.ylabel('Silhouette Score')
-    plt.title('Silhouette Score for optimal K')
+
+    plt.xlabel(xlabel if xlabel is not None else 'Number of Clusters (k)')
+    plt.ylabel(ylabel if ylabel is not None else 'Silhouette Score')
+    plt.title(title if title is not None else 'Silhouette Score for optimal K')
     plt.legend()
     plt.grid(True)
-    plt.savefig(outfile_path+'sillohuette_plot', dpi=300)
+    plt.savefig(outfile_path + 'sillohuette_plot', dpi=300)
     plt.close()
-
     return optimal_k_sil
- 
-def plot_elbow_scores(cluster_range, inertia_scores, outfile_path="elbow_method.png"):
-    '''quickly plot sillouhette scores afte running kmeans
+
+def plot_elbow_scores(cluster_range, inertia_scores, outfile_path="elbow_method.png",
+                      title=None, xlabel=None, ylabel=None):
+    """
+    Plot inertia over k and mark elbow via second derivative.
+
     Parameters
     ----------
+    cluster_range : array-like
+        k values evaluated.
+
+    inertia_scores : array-like
+        KMeans inertia per k.
+
+    outfile_path : str, default="elbow_method.png"
+        Path prefix where figure is saved (suffix 'elbow_plot' is appended).
+
+    title, xlabel, ylabel : str or None
+        Optional title/axis labels.
 
     Returns
     -------
+    optimal_k : int
+        Estimated elbow k.
+    """
+    diff = np.diff(inertia_scores)
+    diff2 = np.diff(diff)
+    optimal_k = cluster_range[np.argmin(diff2) + 1]
 
-    Notes
-    -----
-
-
-    Examples
-    --------
-
-    '''
-
-    # Find the elbow point using the difference in inertia
-    diff = np.diff(inertia_scores)  # First derivative
-    diff2 = np.diff(diff)  # Second derivative (change in slope)
-    optimal_k = cluster_range[np.argmin(diff2)+1]  # +1 because diff2 is one step shorter
-
-    # Plot the Elbow Method
     plt.figure(figsize=(8, 5))
     plt.plot(cluster_range, inertia_scores, marker='o', linestyle='-')
     plt.axvline(optimal_k, color='red', linestyle='--', linewidth=2, label=f'Optimal k = {optimal_k}')
 
-    plt.xlabel('Number of Clusters (k)')
-    plt.ylabel('Inertia (Sum of Squared Distances)')
-    plt.title('Elbow Method for Optimal k')
+    plt.xlabel(xlabel if xlabel is not None else 'Number of Clusters (k)')
+    plt.ylabel(ylabel if ylabel is not None else 'Inertia (Sum of Squared Distances)')
+    plt.title(title if title is not None else 'Elbow Method for Optimal k')
     plt.legend()
     plt.grid(True)
-    plt.savefig(outfile_path+'elbow_plot', dpi=300)  # Save the figure
+    plt.savefig(outfile_path + 'elbow_plot', dpi=300)
     plt.close()
 
-
     return optimal_k
 
 #Circos plots
@@ -495,18 +619,21 @@ def create_2d_color_mappings(labels=([80]*20)+([160]*10),
         sample_color_mappings = [label_dict[i] for i in labels]
         return sample_color_mappings
 
-def visualize_reduction(embedding_coordinates, color_mappings=None, 
-                  custom=False, 
-                  savepath=os.getcwd(), 
-                  title="Dimensional Reduction of (PCA) of GCU and CGU Systems", 
-                  colors_list=['purple', 'orange', 'green', 'yellow', 'blue', 'red', 'pink', 'cyan', 'grey','brown'],
-                  cmap=None,
-                  legend_labels=None,
-                  axis_one_label=None,
-                  axis_two_label=None):
+def visualize_reduction(embedding_coordinates, 
+                        color_mappings=None, 
+                        savepath=os.getcwd(), 
+                        title=None, 
+                        cmap=None,
+                        axis_one_label=None,
+                        axis_two_label=None,
+                        cbar_label=None,
+                        gridvisible=False):
 
     axis_one_label=None if axis_one_label is not None else 'Embedding Space Axis 1'
     axis_two_label=None if axis_two_label is not None else 'Embedding Space Axis 2'
+    title=title if title is not None else "Dimensional Reduction of Systems"
+    cbar_label=cbar_label if cbar_label is not None else "Value"
+    cmap = cmap if cmap is not None else cm.magma_r
 
     labels_font_dict = {
         'family': 'monospace',
@@ -519,47 +646,26 @@ def visualize_reduction(embedding_coordinates, color_mappings=None,
     fig = plt.figure(figsize=(16, 12), dpi=300)
     ax = plt.gca()
 
+
+    if color_mappings is not None and len(color_mappings) > 0:
+        scatter=ax.scatter(embedding_coordinates[:, 0], embedding_coordinates[:, 1],
+                            c=color_mappings, cmap=cmap, alpha=0.6)
+        add_custom_colorbar(scatter, color_mappings, cbar_label, plt.gca(), cmap=cmap)
+ 
     if color_mappings is None or len(color_mappings) == 0:
         color_mappings = np.arange(embedding_coordinates.shape[0])
-        custom = False
-        legend_labels = None
         print("No color_mappings provided — defaulting to sample index gradient.")
-    
-    unique_vals = np.unique(color_mappings)
-    
-    if custom:
-        # Discrete category color mapping
-        scatter = ax.scatter(embedding_coordinates[:, 0], embedding_coordinates[:, 1], c=color_mappings, 
-                             cmap=ListedColormap(colors_list[:len(unique_vals)]), alpha=0.6)
-        
-        if legend_labels is not None:
-            legend_handles = [plt.Line2D([0], [0], marker='o', color='w', markersize=10, 
-                                         markerfacecolor=color, label=label) 
-                              for label, color in legend_labels.items()]
-            ax.legend(handles=legend_handles, title="System Types", loc="upper right", prop={'size': 20, 'weight': 'bold'})
+        scatter=ax.scatter(embedding_coordinates[:, 0], embedding_coordinates[:, 1],
+                            c=color_mappings, cmap=cmap, alpha=0.6)
+        add_continuous_colorbar(scatter, color_mappings, cbar_label, plt.gca(), cmap=cmap)
 
-    else:
-        # Use provided colormap, fallback to red
-        norm = Normalize(vmin=np.min(color_mappings), vmax=np.max(color_mappings))
-        cmap = cmap if cmap is not None else plt.get_cmap('Reds')
-
-        ax.scatter(embedding_coordinates[:, 0], embedding_coordinates[:, 1],
-                             c=color_mappings, cmap=cmap, norm=norm, alpha=0.6)
-
-        cbar_ticks = np.linspace(np.min(color_mappings), np.max(color_mappings), 10, dtype=int)
-        
-        cbar = plt.colorbar(cm.ScalarMappable(norm=norm, cmap=cmap), ax=ax,
-                            ticks=cbar_ticks, shrink=0.8, aspect=30, pad=0.02)
-        
-        cbar.set_label("Value", fontdict=labels_font_dict,
-                       rotation=270, labelpad=25)
-        
-        cbar.ax.set_yticklabels([str(t) for t in cbar_ticks])
 
     # Final touches
     for spine in ax.spines.values():
         spine.set_visible(False)
 
+    ax.grid(visible=gridvisible)
+    set_ticks(ax=plt.gca())
     ax.set_title(title, fontdict=labels_font_dict)
     ax.set_xlabel(axis_one_label, fontdict=labels_font_dict)
     ax.set_ylabel(axis_two_label, fontdict=labels_font_dict)
@@ -570,62 +676,94 @@ def visualize_reduction(embedding_coordinates, color_mappings=None,
     plt.savefig(savepath, dpi=500)
     plt.close()
 
-def highlight_reps_in_embeddingspace(reduced_coordinates,
-                    frame_list=((([80] * 20) + ([160] * 10)) * 2),
-                    outfilepath='/zfshomes/lperez/thesis_figures/PCA/test_one_rep',
-                    cmap=cm.magma_r):
-    '''Visualizes and saves a replicates inside of embedding space
+    return
+
+#RMSD lineplots
+def rmsd_lineplots(pandasdf=None,title='RMSD plot',
+                xgroupvar='window',
+                ygroupvar='rmsd',
+                xlab="window",
+                ylab="rmsd",
+                groupingvar='cluster',
+                cmap=cm.inferno_r,
+                legendtitle='Cluster',
+                outfilepath=os.getcwd()):
+    '''
+    Creates a line plot of RMSD values across a specified grouping variable.
 
     Parameters
     ----------
-    X_pca : np.ndarray, shape=(n_samples, n_components)
-        The results of fitting a PCA analysis and using the .transform() method.
+    pandasdf : pandas.DataFrame, default=None
+        A DataFrame containing the values to be plotted. It should contain at least 
+        the columns specified by `xgroupvar`, `ygroupvar`, and `groupingvar`.
 
-    frame_list : list of int, optional
-        A list holding integer counts of the number of frames in each replicate. 
-        Default is (([80] * 20) + ([160] * 10)) * 2.
-    
-    cmap:matplotlib.cm.cmap:,default=magma_r
-        colormap of choice for highlighting replicates in embeddingspace
+    title : str, default='RMSD plot'
+        Title for the plot.
+
+    xgroupvar : str, default='window'
+        Column name in `pandasdf` to be used as the x-axis variable.
+
+    ygroupvar : str, default='rmsd'
+        Column name in `pandasdf` to be used as the y-axis variable.
+
+    xlab : str, default='window'
+        Label for the x-axis.
+
+    ylab : str, default='rmsd'
+        Label for the y-axis.
+
+    groupingvar : str, default='cluster'
+        Column name in `pandasdf` used to group lines (e.g., clusters or categories) and
+        Column name used by seaborn to color lines by category.
+
+    palette : colormap or palette, default=cm.magma_r
+        Color mapping for different groups. Accepts matplotlib colormap or seaborn palette.
+
+    legendtitle : str, default='Cluster'
+        Title for the legend.
+
+    outfilepath : str, default=os.getcwd()
+        Path prefix where the figure will be saved. The function appends '_rmsdlineplot' to this path.
 
     Returns
     -------
     None
+        Saves the line plot to disk and displays it.
 
     Notes
     -----
-    Each replicate is plotted in its own subplot. A new row of plots begins every 30 replicates.
+    This function uses seaborn's lineplot for grouped visualization of RMSD trajectories 
+    or similar metrics. It assumes the input DataFrame has one row per observation.
+
+    Examples
+    --------
     '''
 
-    import numpy as np
-    import matplotlib.pyplot as plt
-    from matplotlib import cm
+    plt.figure(figsize=(10, 8))
+    sns.lineplot(
+        data=pandasdf,
+        x=xgroupvar,
+        y=ygroupvar,
+        hue=groupingvar,   # automatically treats as categorical
+        palette=cmap  # choose color palette
+    )
+    ax=plt.gca()
 
-    rep_iterator = 0
-    
-    for entry in range(len(frame_list)):
+    for spine in ax.spines.values():
+        spine.set_visible(False)
 
-        colors = np.full(reduced_coordinates.shape[0], 'yellow')
-        colors[rep_iterator:rep_iterator+frame_list[entry]] = 'blue'  
-
-        # ticks for scaling
-        x_min, x_max = reduced_coordinates[:, 0].min(), reduced_coordinates[:, 0].max()
-        y_min, y_max = reduced_coordinates[:, 1].min(), reduced_coordinates[:, 1].max()
-        plt.xticks(np.arange(np.floor(x_min), np.ceil(x_max) + 1, 1))
-        plt.yticks(np.arange(np.floor(y_min), np.ceil(y_max) + 1, 1))
-
-        plt.scatter(reduced_coordinates[:,0],reduced_coordinates[:,1],c=colors,s=5,cmap=cmap)
-        plt.grid(visible=False)
-        plt.savefig(f"{outfilepath}_rep{entry}.png")
-        plt.close()
-        
-        rep_iterator+=frame_list[entry]
+    plt.legend(title=legendtitle, bbox_to_anchor=(1.0, 1), loc="upper left")    
+    plt.xlabel(xlab)
+    plt.ylabel(ylab)
+    plt.savefig(outfilepath+'_rmsdlineplot',dpi=800)
     
+    plt.close()
     return
 
+
 #Contour plots 
 def contour_embedding_space(outfile_path, embeddingspace_coordinates, levels=10, thresh=0, bw_adjust=.5,
-                             title=None, xlabel=None, ylabel=None):
+                             title=None, xlabel=None, ylabel=None,gridvisible=False):
     '''Plots a contour map of embedding space coordinates.
 
     Parameters
@@ -675,6 +813,7 @@ def contour_embedding_space(outfile_path, embeddingspace_coordinates, levels=10,
     '''
 
     outfile_path = outfile_path if outfile_path is not None else os.getcwd()
+    gridvisible = gridvisible if gridvisible is not None else False
 
     sns.kdeplot(
         x=embeddingspace_coordinates[:, 0],
@@ -688,6 +827,7 @@ def contour_embedding_space(outfile_path, embeddingspace_coordinates, levels=10,
     )
 
     ax = plt.gca()
+    
     for spine in ax.spines.values():
         spine.set_visible(False)
 
@@ -698,7 +838,7 @@ def contour_embedding_space(outfile_path, embeddingspace_coordinates, levels=10,
     if ylabel:
         plt.ylabel(ylabel)
 
-    plt.grid(visible=False)
+    plt.grid(visible=gridvisible)
     plt.savefig(outfile_path, dpi=800)
     plt.close()
 
